@@ -8,9 +8,6 @@ returns integer as $$
 --      It moves small amount at a time.  It should be called
 --      until it returns 0
 --
--- Parameters:
---      arg - desc
---
 -- Returns:
 --      Number of events processed.
 -- ----------------------------------------------------------------------
@@ -20,10 +17,9 @@ declare
 begin
     cnt := 0;
     for rec in
-        select pgq.insert_event_raw(queue_name,
-                    ev_id, ev_time, ev_owner, ev_retry, ev_type, ev_data,
-                    ev_extra1, ev_extra2, ev_extra3, ev_extra4),
-               ev_owner, ev_id
+        select queue_name,
+               ev_id, ev_time, ev_owner, ev_retry, ev_type, ev_data,
+               ev_extra1, ev_extra2, ev_extra3, ev_extra4
           from pgq.retry_queue, pgq.queue, pgq.subscription
          where ev_retry_after <= current_timestamp
            and sub_id = ev_owner
@@ -32,6 +28,10 @@ begin
          limit 10
     loop
         cnt := cnt + 1;
+        perform pgq.insert_event_raw(rec.queue_name,
+                    rec.ev_id, rec.ev_time, rec.ev_owner, rec.ev_retry,
+                    rec.ev_type, rec.ev_data, rec.ev_extra1, rec.ev_extra2,
+                    rec.ev_extra3, rec.ev_extra4);
         delete from pgq.retry_queue
          where ev_owner = rec.ev_owner
            and ev_id = rec.ev_id;
